'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.loadConfig = exports.preloadModule = exports.getModuleRequirePath = exports.help = exports.getScriptsAndArgs = undefined;

var _path = require('path');

var _lodash = require('lodash.remove');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.contains');

var _lodash4 = _interopRequireDefault(_lodash3);

var _lodash5 = require('lodash.isplainobject');

var _lodash6 = _interopRequireDefault(_lodash5);

var _shellEscape = require('shell-escape');

var _shellEscape2 = _interopRequireDefault(_shellEscape);

var _lodash7 = require('lodash.isempty');

var _lodash8 = _interopRequireDefault(_lodash7);

var _safe = require('colors/safe');

var _safe2 = _interopRequireDefault(_safe);

var _getLogger = require('./get-logger');

var _getLogger2 = _interopRequireDefault(_getLogger);

var _resolveScriptObjectToString = require('./resolve-script-object-to-string');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var log = (0, _getLogger2.default)();

/**
 * Attempts to load the given module. This is used for the --require functionality of the CLI
 * @param  {String} moduleName The module to attempt to require
 * @return {*} The required module
 */
var preloadModule = getAttemptModuleRequireFn(function (moduleName, requirePath) {
  log.warn({
    message: _safe2.default.yellow('Unable to preload "' + moduleName + '". Attempted to require as "' + requirePath + '"'),
    ref: 'unable-to-preload-module'
  });
  return undefined;
});

/**
 * Attempts to load the config and logs an error if there's a problem
 * @param  {String} configPath The path to attempt to require the config from
 * @return {*} The required module
 */
var loadConfig = getAttemptModuleRequireFn(function onFail(configPath, requirePath) {
  log.error({
    message: _safe2.default.red('Unable to find config at "' + configPath + '". Attempted to require as "' + requirePath + '"'),
    ref: 'unable-to-find-config'
  });
  return undefined;
});

exports.getScriptsAndArgs = getScriptsAndArgs;
exports.help = help;
exports.getModuleRequirePath = getModuleRequirePath;
exports.preloadModule = preloadModule;
exports.loadConfig = loadConfig;

/****** implementations ******/

function getScriptsAndArgs(program) {
  var scripts = void 0,
      args = void 0,
      parallel = void 0;
  if (!(0, _lodash8.default)(program.parallel)) {
    scripts = program.parallel.split(',');
    args = getArgs(program.args, program.rawArgs, scripts);
    parallel = true;
  } else {
    scripts = program.args[0].split(',');
    args = getArgs(program.args.slice(1), program.rawArgs, scripts);
    parallel = false;
  }
  return { scripts: scripts, args: args, parallel: parallel };
}

function getArgs(args, rawArgs, scripts) {
  var allArgs = rawArgs.slice(2);
  var psArgs = ['-p', '--parallel', '-c', '--config', '-r', '--require'];
  var psFlags = ['-s', '--silent'];
  var cleanedArgs = (0, _lodash2.default)(allArgs, function (item, index, arry) {
    var isArgOrFlag = (0, _lodash4.default)(psArgs, item) || (0, _lodash4.default)(psFlags, item);
    var isArgValue = (0, _lodash4.default)(psArgs, arry[index - 1]);
    var isInScripts = (0, _lodash4.default)(scripts, item);
    return !isArgOrFlag && !isArgValue && !isInScripts;
  });
  return (0, _shellEscape2.default)(cleanedArgs);
}

/**
 * Determines the proper require path for a module. If the path starts with `.` then it is resolved with process.cwd()
 * @param  {String} moduleName The module path
 * @return {String} the module path to require
 */
function getModuleRequirePath(moduleName) {
  return moduleName[0] === '.' ? (0, _path.resolve)(process.cwd(), moduleName) : moduleName;
}

function getAttemptModuleRequireFn(onFail) {
  return function attemptModuleRequire(moduleName) {
    var requirePath = getModuleRequirePath(moduleName);
    try {
      return requireDefaultFromModule(requirePath);
    } catch (e) {
      if (e instanceof SyntaxError) {
        throw e;
      }
      return onFail(moduleName, requirePath);
    }
  };
}

/**
 * Requires the given module and returns the `default` if it's an `__esModule`
 * @param  {String} modulePath The module to require
 * @return {*} The required module (or it's `default` if it's an `__esModule`)
 */
function requireDefaultFromModule(modulePath) {
  var mod = require(modulePath);
  if (mod.__esModule) {
    return mod.default;
  } else {
    return mod;
  }
}

function help(_ref) {
  var scripts = _ref.scripts;

  var availableScripts = getAvailableScripts(scripts);
  var scriptLines = availableScripts.map(function (_ref2) {
    var name = _ref2.name;
    var description = _ref2.description;
    var script = _ref2.script;

    var coloredName = _safe2.default.green(name);
    var coloredScript = _safe2.default.gray(script);
    var line = void 0;
    if (description) {
      line = [coloredName, _safe2.default.white(description), coloredScript];
    } else {
      line = [coloredName, coloredScript];
    }
    return line.join(' - ').trim();
  });
  if (!scriptLines.length) {
    return _safe2.default.yellow('There are no scripts available');
  } else {
    var topMessage = 'Available scripts (camel or kebab case accepted)';
    var message = topMessage + '\n\n' + scriptLines.join('\n');
    return message;
  }
}

function getAvailableScripts(config) {
  var prefix = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];

  var excluded = ['description', 'script', 'default'];
  return Object.keys(config).reduce(function (scripts, key) {
    var val = config[key];
    if ((0, _lodash4.default)(excluded, key)) {
      return scripts;
    }
    var scriptObj = (0, _resolveScriptObjectToString.resolveScriptObjectToScript)(val);
    var prefixed = [].concat(_toConsumableArray(prefix), [key]);
    if (scriptObj) {
      var description = scriptObj.description;
      var script = scriptObj.script;

      scripts = [].concat(_toConsumableArray(scripts), [{ name: prefixed.join('.'), description: description, script: script }]);
    }
    if ((0, _lodash6.default)(val)) {
      return [].concat(_toConsumableArray(scripts), _toConsumableArray(getAvailableScripts(val, prefixed)));
    }
    return scripts;
  }, []);
}